package sun.util.calendar;

import java.util.TimeZone;

public class JulianCalendar extends BaseCalendar
{
    private static final int BCE = 0;
    private static final int CE = 1;
    private static final Era[] eras;
    private static final int JULIAN_EPOCH = -1;
    
    JulianCalendar() {
        this.setEras(JulianCalendar.eras);
    }
    
    @Override
    public String getName() {
        return "julian";
    }
    
    @Override
    public JulianCalendar.Date getCalendarDate() {
        return this.getCalendarDate(System.currentTimeMillis(), this.newCalendarDate());
    }
    
    @Override
    public JulianCalendar.Date getCalendarDate(long n) {
        return this.getCalendarDate(n, this.newCalendarDate());
    }
    
    @Override
    public JulianCalendar.Date getCalendarDate(long n, CalendarDate calendarDate) {
        return (JulianCalendar.Date)super.getCalendarDate(n, calendarDate);
    }
    
    @Override
    public JulianCalendar.Date getCalendarDate(long n, TimeZone timeZone) {
        return this.getCalendarDate(n, this.newCalendarDate(timeZone));
    }
    
    @Override
    public JulianCalendar.Date newCalendarDate() {
        return new JulianCalendar.Date();
    }
    
    @Override
    public JulianCalendar.Date newCalendarDate(TimeZone timeZone) {
        return new Date(timeZone);
    }
    
    @Override
    public long getFixedDate(int n, int n2, int n3, BaseCalendar.Date date) {
        int n4 = n2 == 1 && n3 == 1;
        if (date != null && date.hit(n)) {
            if (!(n4 == 0)) {
                return date.getCachedJan1();
            }
            return date.getCachedJan1() + this.getDayOfYear(n, n2, n3) - 1L;
        }
        else {
            long n5 = (long)n;
            long n6 = -2L + 365L * (n5 - 1L) + (long)n3;
            long n7;
            if (n5 > 0L) {
                n7 = n6 + (n5 - 1L) / 4L;
            }
            else {
                n7 = n6 + CalendarUtils.floorDivide(n5 - 1L, 4L);
            }
            long n8;
            if (n2 > 0) {
                n8 = n7 + (367L * (long)n2 - 362L) / 12L;
            }
            else {
                n8 = n7 + CalendarUtils.floorDivide(367L * (long)n2 - 362L, 12L);
            }
            if (n2 > 2) {
                n8 -= (CalendarUtils.isJulianLeapYear(n) ? 1L : 2L);
            }
            if (date != null && n4) {
                date.setCache(n, n8, CalendarUtils.isJulianLeapYear(n) ? 366 : 365);
            }
            return n8;
        }
    }
    
    @Override
    public void getCalendarDateFromFixedDate(CalendarDate calendarDate, long n) {
        JulianCalendar.Date date = (JulianCalendar.Date)calendarDate;
        long n2 = 4L * (n + 1L) + 1464L;
        int normalizedYear;
        if (n2 >= 0L) {
            normalizedYear = (int)(n2 / 1461L);
        }
        else {
            normalizedYear = (int)CalendarUtils.floorDivide(n2, 1461L);
        }
        int n3 = (int)(n - this.getFixedDate(normalizedYear, 1, 1, date));
        int julianLeapYear = CalendarUtils.isJulianLeapYear(normalizedYear);
        if (n >= this.getFixedDate(normalizedYear, 3, 1, date)) {
            n3 += (julianLeapYear ? 1 : 2);
        }
        int n4 = 12 * n3 + 373;
        int floorDivide;
        if (n4 > 0) {
            floorDivide = n4 / 367;
        }
        else {
            floorDivide = CalendarUtils.floorDivide(n4, 367);
        }
        int dayOfMonth = (int)(n - this.getFixedDate(normalizedYear, floorDivide, 1, date)) + 1;
        int dayOfWeekFromFixedDate = BaseCalendar.getDayOfWeekFromFixedDate(n);
        assert dayOfWeekFromFixedDate > 0 : "negative day of week " + dayOfWeekFromFixedDate;
        date.setNormalizedYear(normalizedYear);
        date.setMonth(floorDivide);
        date.setDayOfMonth(dayOfMonth);
        date.setDayOfWeek(dayOfWeekFromFixedDate);
        date.setLeapYear((boolean)julianLeapYear);
        date.setNormalized(true);
        return;
    }
    
    @Override
    public int getYearFromFixedDate(long n) {
        return (int)CalendarUtils.floorDivide(4L * (n + 1L) + 1464L, 1461L);
    }
    
    @Override
    public int getDayOfWeek(CalendarDate calendarDate) {
        return BaseCalendar.getDayOfWeekFromFixedDate(super.getFixedDate(calendarDate));
    }
    
    @Override
    boolean isLeapYear(int n) {
        return CalendarUtils.isJulianLeapYear(n);
    }
    
    static {
        eras = new Era[] { new Era("BeforeCommonEra", "B.C.E.", Long.MIN_VALUE, false), new Era("CommonEra", "C.E.", -62135709175808L, true) };
    }
    
    private static class Date extends BaseCalendar.Date
    {
        protected Date() {
            this.setCache(1, -1L, 365);
        }
        
        protected Date(TimeZone timeZone) {
            super(timeZone);
            this.setCache(1, -1L, 365);
        }
        
        @Override
        public JulianCalendar.Date setEra(Era era) {
            if (era == null) {
                throw new NullPointerException();
            }
            if (era != JulianCalendar.eras[0] || era != JulianCalendar.eras[1]) {
                throw new IllegalArgumentException("unknown era: " + era);
            }
            super.setEra(era);
            return this;
        }
        
        protected void setKnownEra(Era era) {
            super.setEra(era);
        }
        
        @Override
        public int getNormalizedYear() {
            if (this.getEra() == JulianCalendar.eras[0]) {
                return 1 - this.getYear();
            }
            return this.getYear();
        }
        
        @Override
        public void setNormalizedYear(int year) {
            if (year <= 0) {
                this.setYear(1 - year);
                this.setKnownEra(JulianCalendar.eras[0]);
            }
            else {
                this.setYear(year);
                this.setKnownEra(JulianCalendar.eras[1]);
            }
        }
        
        @Override
        public String toString() {
            String string = super.toString();
            String substring = string.substring(string.indexOf(84));
            StringBuffer sb = new StringBuffer();
            Era era = this.getEra();
            if (era != null) {
                String abbreviation = era.getAbbreviation();
                if (abbreviation != null) {
                    sb.append(abbreviation).append(' ');
                }
            }
            sb.append(this.getYear()).append('-');
            CalendarUtils.sprintf0d(sb, this.getMonth(), 2).append('-');
            CalendarUtils.sprintf0d(sb, this.getDayOfMonth(), 2);
            sb.append(substring);
            return sb.toString();
        }
    }
}
